/**
 * 浏览器截图工具类
 * 用于OCR识别备选方案
 */

/**
 * 在浏览器中截取屏幕区域
 * 注意：仅在支持屏幕捕获API的浏览器中有效
 * 
 * @returns 截图的base64字符串
 */
export async function captureScreen(): Promise<string> {
  try {
    // 请求屏幕共享
    const mediaStream = await navigator.mediaDevices.getDisplayMedia({
      video: {
        displaySurface: 'monitor'
      },
      audio: false
    });
    
    // 获取视频轨道
    const videoTrack = mediaStream.getVideoTracks()[0];
    
    // 创建一个视频元素来显示流
    const video = document.createElement('video');
    video.srcObject = mediaStream;
    
    // 当视频加载完毕后进行截图
    return new Promise<string>((resolve, reject) => {
      video.onloadedmetadata = () => {
        // 设置视频尺寸
        video.width = video.videoWidth;
        video.height = video.videoHeight;
        
        // 播放视频以便捕获当前帧
        video.play();
        
        // 创建canvas元素用于截图
        const canvas = document.createElement('canvas');
        canvas.width = video.width;
        canvas.height = video.height;
        
        // 在canvas上绘制视频当前帧
        const ctx = canvas.getContext('2d');
        if (!ctx) {
          reject(new Error('无法获取canvas上下文'));
          return;
        }
        
        // 绘制视频帧到canvas
        setTimeout(() => {
          try {
            ctx.drawImage(video, 0, 0, canvas.width, canvas.height);
            
            // 停止所有轨道
            mediaStream.getTracks().forEach(track => track.stop());
            
            // 将canvas转换为base64字符串
            const dataUrl = canvas.toDataURL('image/png');
            resolve(dataUrl);
          } catch (error) {
            reject(error);
          }
        }, 200);
      };
      
      video.onerror = () => {
        reject(new Error('视频加载失败'));
      };
    });
  } catch (error) {
    console.error('截图失败:', error);
    throw error;
  }
}

/**
 * 处理图像以提高OCR识别率
 * 基于Data URL的图像处理
 * 
 * @param imageDataUrl 图像的Data URL
 * @returns 处理后的Data URL
 */
export async function preprocessImageForOCR(imageDataUrl: string): Promise<string> {
  return new Promise((resolve, reject) => {
    const img = new Image();
    img.onload = () => {
      // 创建canvas
      const canvas = document.createElement('canvas');
      const ctx = canvas.getContext('2d');
      if (!ctx) {
        reject(new Error('无法获取canvas上下文'));
        return;
      }
      
      // 设置canvas尺寸
      canvas.width = img.width;
      canvas.height = img.height;
      
      // 绘制原图到canvas
      ctx.drawImage(img, 0, 0);
      
      // 获取图像数据
      const imageData = ctx.getImageData(0, 0, canvas.width, canvas.height);
      const data = imageData.data;
      
      // 简单的图像预处理 - 增强对比度和边缘
      for (let i = 0; i < data.length; i += 4) {
        // 计算灰度值
        const r = data[i];
        const g = data[i + 1];
        const b = data[i + 2];
        let gray = 0.299 * r + 0.587 * g + 0.114 * b;
        
        // 应用阈值处理以增强对比度
        // 根据灰度值应用阈值处理，得到二值化图像
        const threshold = 180;
        gray = gray > threshold ? 255 : 0;
        
        // 将处理后的值写回
        data[i] = data[i + 1] = data[i + 2] = gray;
      }
      
      // 将处理后的图像数据写回canvas
      ctx.putImageData(imageData, 0, 0);
      
      // 转换为data URL并返回
      const processedDataUrl = canvas.toDataURL('image/png');
      resolve(processedDataUrl);
    };
    
    img.onerror = () => {
      reject(new Error('图像加载失败'));
    };
    
    img.src = imageDataUrl;
  });
}

/**
 * 将data URL转换为Blob对象
 * 
 * @param dataURL 图像的Data URL
 * @returns Blob对象
 */
export function dataURLtoBlob(dataURL: string): Blob {
  const parts = dataURL.split(';base64,');
  const contentType = parts[0].split(':')[1];
  const raw = window.atob(parts[1]);
  const rawLength = raw.length;
  const uInt8Array = new Uint8Array(rawLength);
  
  for (let i = 0; i < rawLength; ++i) {
    uInt8Array[i] = raw.charCodeAt(i);
  }
  
  return new Blob([uInt8Array], { type: contentType });
} 